@@ -2,11 +2,9 @@
 #             Note that earth.cv returns null unless nfold > 1.
 
 earth.cv <- function(object, x, y,
-                     subset, weights, na.action, keepxy, trace, glm,
-                     ncross, nfold, stratify,
-                     varmod.method=c("none", VARMOD.METHODS),
-                     varmod.exponent, varmod.conv, varmod.clamp, varmod.minspan,
-                     Scale.y, env, degree=1, ...)
+    subset, weights, na.action, pmethod, keepxy, trace, glm, degree, nprune,
+    ncross, nfold, stratify, get.oof.fit.tab, get.oof.rsq.per.subset,
+    Scale.y, env, ...)
 {
     get.fold.rsq.per.subset <- function(foldmod, oof.y, max.nterms, trace, must.print.dots)
     {
@@ -15,11 +13,11 @@ earth.cv <- function(object, x, y,
         # nrow(foldmod$dirs) is the number of terms in this fold's model before pruning
         for(nterms in 1:min(max.nterms, nrow(foldmod$dirs))) {
             trace.get.fold1(trace, must.print.dots, nterms)
-            # penalty=-1 to enforce strict nprune
+            # penalty=-1 to enforce strict nprune TODO consider changing to pmethod=none
             # glm=NULL for speed, ok because we don't need the glm submodel
             # TODO with keepxy=TRUE, 70% of cv time is spent in update.earth
-            pruned.foldmod <- update(foldmod, nprune=nterms, penalty=-1, ponly=TRUE,
-                                     glm=NULL, trace=max(0, trace-1))
+            pruned.foldmod <- update(foldmod, nprune=min(nprune, nterms),
+                    penalty=-1, ponly=TRUE, glm=NULL, trace=max(0, trace-1))
             fit <- predict(pruned.foldmod, newdata=x, type="earth")
             oof.fit    <- fit[oof.subset,  , drop=FALSE]
             infold.fit <- fit[infold.subset, , drop=FALSE]
@@ -37,16 +35,16 @@ earth.cv <- function(object, x, y,
                     oof.rsq.per.subset[nterms] <- infold.rsq.per.subset[nterms] <- NA
         }
         trace.get.fold2(trace, must.print.dots, nterms)
-        list(oof.rsq.per.subset=oof.rsq.per.subset,
-             infold.rsq.per.subset=infold.rsq.per.subset)
+        list(oof.rsq.per.subset    = oof.rsq.per.subset,
+             infold.rsq.per.subset = infold.rsq.per.subset)
     }
     #--- earth.cv starts here ---
+    # We called check.cv.args(ncross, nfold, varmod.method, pmethod)
+    # earlier so it's safe to use those args here
+    # Likewise, subset arg was already checked in earth.fit.
     stratify <- check.boolean(stratify)
-    # check varmod.method is legal and expand
-    varmod.method <- match.arg1(varmod.method, "varmod.method")
-    check.cv.args(ncross, nfold, varmod.method)
-    if(ncross < 1 || nfold <= 1)
-        return (NULL)
+    stopifnot(ncross >= 1, nfold > 1)
+    trace1(trace, "\n")
     if(nfold > nrow(x))
         nfold <- nrow(x)
     if(!is.null(object$glm.bpairs))
@@ -55,13 +53,15 @@ earth.cv <- function(object, x, y,
     wp <- wp.expanded <- object$wp
     if(is.null(wp))
         wp.expanded <- repl(1, ncol(y)) # all ones vector
-    list. <- list()                     # returned list of cross validated models
+    cv.list <- list()                   # returned list of cross validated models
     ncases <- nrow(x)
     nresp <- ncol(y)                    # number of responses
     # ndigits aligns trace prints without too much white space
     ndigits <- ceiling(log10(ncases - ncases/nfold + .1))
     # print pacifier dots if get.fold.rss.per.subset will be slow
-    must.print.dots <- trace >= .5 && trace <= 1 && keepxy && nrow(x) * max.nterms > 50e3
+    must.print.dots <- trace >= .5 && trace <= 1 &&
+        (get.oof.rsq.per.subset || get.oof.fit.tab) && 
+        nrow(x) * max.nterms > 50e3
     trace.cv.header(object, nresp, trace, must.print.dots)
     n.oof.digits <- ceiling(log10(1.2 * ncases / nfold)) # 1.2 allows for diff sized subsets
     groups <- matrix(NA, nrow=ncross*ncases, ncol=2)
@@ -91,9 +91,10 @@ earth.cv <- function(object, x, y,
     resp.names.plus.mean <- c(colnames(y), "mean") # response names plus "mean"
     resp.names.plus.max  <- c(colnames(y), "max")
     ncross.fold <- ncross * nfold
-    nvars <- double(ncross.fold+1)    # number of used predictors in each CV model
-    nterms <- double(ncross.fold+1)   # number of selected terms in each CV model
-    names(nvars) <- names(nterms) <- fold.names.plus.mean
+    nvars.selected.by.gcv <- double(ncross.fold+1)  # nbr of used predictors in each CV mod
+    nterms.selected.by.gcv <- double(ncross.fold+1) # nbr of selected terms in each CV mod
+    names(nvars.selected.by.gcv) <- fold.names.plus.mean
+    names(nterms.selected.by.gcv) <- fold.names.plus.mean
 
     rsq.tab <- matrix(0, nrow=ncross.fold+1, ncol=1+nresp) # table of cv results, +1 for means
     colnames(rsq.tab) <- resp.names.plus.mean
@@ -120,12 +121,12 @@ earth.cv <- function(object, x, y,
         rownames(maxerr.tab) <- fold.names.plus.all
     }
     oof.rsq.tab <- infold.rsq.tab <- oof.fit.tab <- NULL
-    if(keepxy) {
+    if(get.oof.rsq.per.subset) {
         oof.rsq.tab <- infold.rsq.tab <- matrix(0, nrow=ncross.fold+1, ncol=max.nterms)
         colnames(oof.rsq.tab) <- colnames(infold.rsq.tab) <- paste0("nterms", 1:max.nterms)
         rownames(oof.rsq.tab) <- rownames(infold.rsq.tab) <- fold.names.plus.all
     }
-    if(varmod.method != "none") {
+    if(get.oof.fit.tab) {
         oof.fit.tab <- matrix(0, nrow=nrow(x), ncol=ncross) # preds on oof data
         colnames(oof.fit.tab) <- paste0("icross", seq_len(ncross))
     }
@@ -140,8 +141,11 @@ earth.cv <- function(object, x, y,
             infold.y <- y[infold.subset,,drop=FALSE]
             infold.weights <- weights[infold.subset]
             foldmod <- earth.default(x=infold.x, y=infold.y, weights=infold.weights,
-                                     wp=wp, Scale.y=Scale.y, subset=subset,
-                                     glm=glm, trace=trace, degree=degree, ...)
+                wp=wp, Scale.y=Scale.y, subset=subset,
+                trace=trace, glm=glm, degree=degree,
+                pmethod=if(pmethod == "cv") "backward" else pmethod,
+                ncross=0, nfold=0, varmod.method="none",
+                ...)
             foldmod$icross <- icross
             foldmod$ifold <- ifold
             oof.x <- x[oof.subset,,drop=FALSE]
@@ -190,14 +194,16 @@ earth.cv <- function(object, x, y,
                         get.maxerr(oof.y[,iresp] - oof.fit[,iresp])
             } # end for iresp
 
-            nvars[icross.fold] <- get.nused.preds.per.subset(foldmod$dirs, foldmod$selected.terms)
-            nterms[icross.fold] <- length(foldmod$selected.terms)
+            nvars.selected.by.gcv[icross.fold] <-
+                get.nused.preds.per.subset(foldmod$dirs, foldmod$selected.terms)
+            nterms.selected.by.gcv[icross.fold] <-
+                length(foldmod$selected.terms)
             if(must.get.class.rate)
                 class.rate.tab[icross.fold, ] <- get.class.rate(oof.fit.resp, oof.y, object$levels)
-            if(keepxy) {
+            if(get.oof.rsq.per.subset) {
                 temp <-
                     get.fold.rsq.per.subset(foldmod, oof.y, max.nterms, trace, must.print.dots)
-                oof.rsq.tab[icross.fold,]  <- temp$oof.rsq
+                oof.rsq.tab[icross.fold,]    <- temp$oof.rsq
                 infold.rsq.tab[icross.fold,] <- temp$infold.rsq
             }
             # init last column of summary tables
@@ -214,15 +220,16 @@ earth.cv <- function(object, x, y,
                 foldmod$bx <- foldmod$residuals <- foldmod$prune.terms <- NULL
             trace.fold(icross, ifold, trace, y, infold.subset, oof.subset, ncross, ndigits,
                        rsq.tab[icross.fold,], n.oof.digits, must.print.dots)
-            list.[[icross.fold]] <- foldmod
+            cv.list[[icross.fold]] <- foldmod
         } # end for ifold
     } # end for icross
 
     # init last row of summary tables
 
     ilast <- ncross.fold+1 # index of last row in tables
-    nvars [ilast]  <- mean(nvars[-ilast])
-    nterms[ilast]  <- mean(nterms[-ilast])
+    nvars.selected.by.gcv [ilast]  <- mean(nvars.selected.by.gcv[-ilast])
+    nterms.selected.by.gcv[ilast]  <- mean(nterms.selected.by.gcv[-ilast])
+
     rsq.tab   [ilast,] <- colMeans(rsq.tab     [-ilast,])
     maxerr.tab[ilast,] <- get.maxerr(maxerr.tab[-ilast,])
     if(is.binomial || is.poisson) {
@@ -233,46 +240,52 @@ earth.cv <- function(object, x, y,
     }
     if(must.get.class.rate)
         class.rate.tab[ilast,]  <- colMeans(class.rate.tab [-ilast,])
+
     oof.rsq.per.subset <- NULL
-    if(keepxy) {
+    if(get.oof.rsq.per.subset) {
         # there will be NAs in oof.rsq.tab if max terms in a fold is
         # less than max terms in full model
-        oof.rsq.tab[ilast,]  <- colMeans(oof.rsq.tab[-ilast,], na.rm=TRUE)
-        infold.rsq.tab[ilast,] <- colMeans(infold.rsq.tab[-ilast,], na.rm=TRUE)
-    }
-    varmod <- NULL
-    if(varmod.method != "none") {
-        model.var <- apply(oof.fit.tab,  1, var)    # var  of each row of oof.fit.tab
-        model.var <- matrix(model.var, ncol=1)
-        varmod <- varmod(object,
-                         varmod.method, varmod.exponent,
-                         varmod.conv, varmod.clamp, varmod.minspan,
-                         trace, x, y, model.var, degree=degree)
+        oof.rsq.tab[ilast,]    <-
+            col.means.with.special.na.handling(oof.rsq.tab[-ilast,])
+        infold.rsq.tab[ilast,] <-
+            col.means.with.special.na.handling(infold.rsq.tab[-ilast,])
     }
     trace1(trace, "\n")
     trace.fold(icross, -1, trace, y, TRUE, TRUE, ncross, ndigits,
                rsq.tab[ilast,], n.oof.digits, must.print.dots)
     if(trace >= .5)
         cat("\n")
-    names(list.) <- fold.names
-    list(list.           = list.,   # list of earth objects built during cross validation
-         nterms          = nterms,
-         nvars           = nvars,
-         groups          = groups, # groups used for cross validation
-         rsq.tab         = rsq.tab,
-         oof.rsq.tab     = oof.rsq.tab,
-         infold.rsq.tab  = infold.rsq.tab,
-         class.rate.tab  = class.rate.tab,
-         maxerr.tab      = maxerr.tab,
-         auc.tab         = if(is.binomial) test.tab else NULL,
-         cor.tab         = if(is.poisson) test.tab else NULL,
-         deviance.tab    = deviance.tab,
-         calib.int.tab   = calib.int.tab,
-         calib.slope.tab = calib.slope.tab,
-         varmod          = varmod,      # null unless varmod specified
-         oof.fit.tab     = oof.fit.tab) # null unless varmod specified
+    names(cv.list) <- fold.names
+    rv <- list(
+        cv.list = cv.list, # list of earth models built during cross validation
+        nterms.selected.by.gcv = nterms.selected.by.gcv,
+        nvars.selected.by.gcv  = nvars.selected.by.gcv,
+        groups          = groups, # groups used for cross validation
+        rsq.tab         = rsq.tab,
+        maxerr.tab      = maxerr.tab,
+        class.rate.tab  = class.rate.tab,
+        auc.tab         = if(is.binomial) test.tab else NULL,
+        cor.tab         = if(is.poisson)  test.tab else NULL,
+        deviance.tab    = deviance.tab,
+        calib.int.tab   = calib.int.tab,
+        calib.slope.tab = calib.slope.tab,
+        oof.fit.tab     = oof.fit.tab,
+        infold.rsq.tab  = infold.rsq.tab,
+        oof.rsq.tab     = oof.rsq.tab)
+    rv
 }
-check.cv.args <- function(ncross, nfold, varmod.method)
+# Return the mean of each column in tab.
+# NAs are ignored, except that the column mean is NA
+# for columns in which over half the entries are NA.
+
+col.means.with.special.na.handling <- function(tab)
+{
+    means <- colMeans(tab, na.rm=TRUE)
+    nna <- colSums(is.na(tab)) # number of NAs in each column
+    means[nna > nrow(tab) / 2] <- NA
+    means # a vector of column means, some entries may be NA
+}
+check.cv.args <- function(ncross, nfold, pmethod, varmod.method)
 {
     check.integer.scalar(ncross, min=0)
     if(ncross > 1000) # 1000 is arbitrary
@@ -321,7 +334,10 @@ get.this.group <- function(icross, ifold, ncases, groups)
 trace.cv.header <- function(object, nresp, trace, must.print.dots)
 {
     if(trace == .5) {
-        printf("Full model grsq %5.3f  rsq %5.3f\n", object$grsq , object$rsq)
+        if(must.print.dots || nresp > 1)
+            cat("\n")
+        printf("Full model GRSq %5.3f RSq %5.3f, starting cross validation\n",
+               object$grsq , object$rsq)
         if(must.print.dots || nresp > 1)
             cat("\n")
     }
@@ -423,30 +439,37 @@ trace.get.fold2 <- function(trace, must.print.dots, nterms)
 }
 print.cv <- function(x) # called from print.earth for cross validated models
 {
-    get.field <- function(field) round(x[[field]][ilast,], 2)
-    get.sd    <- function(field) round(apply(x[[field]][-ilast,], 2, sd), 3)
+    cv.field <- function(field) round(x[[field]][ilast,], 3)
+
+    cv.sd <- function(field) round(apply(x[[field]][-ilast,], 2, sd), 3)
+
     #--- print.cv starts here ---
-    stopifnot(!is.null(x$cv.list))
+    stopifnot(!is.null(x$cv.list), x$pmethod != "cv")
     ilast <- nrow(x$cv.rsq.tab) # index of "all" row in summary tables
     cat("\nNote: the cross-validation sd's below are standard deviations across folds\n\n")
 
     printf("Cross validation:   nterms %.2f sd %.2f    nvars %.2f sd %.2f\n\n",
-        x$cv.nterms[ilast], sd(x$cv.nterms[-ilast]),
-        x$cv.nvars[ilast], sd(x$cv.nvars[-ilast]))
+        x$cv.nterms.selected.by.gcv[ilast],
+        sd(x$cv.nterms.selected.by.gcv[-ilast]),
+        x$cv.nvars.selected.by.gcv[ilast],
+        sd(x$cv.nvars.selected.by.gcv[-ilast]))
 
-    wide.spacing <- is.null(x$cv.deviance.tab) # if printing little then use wide spacing
+    # if printing little then use wide spacing
+    wide.spacing <- is.null(x$cv.deviance.tab)
 
     # create a data.frame and print that
-    tab <- if(wide.spacing) data.frame("    CVRSq"=get.field("cv.rsq.tab"), check.names = FALSE)
-           else             data.frame("CVRSq"  =get.field("cv.rsq.tab"))
+    tab <- if(wide.spacing)
+               data.frame("    CVRSq"=cv.field("cv.rsq.tab"), check.names = FALSE)
+           else
+               data.frame("CVRSq"  =cv.field("cv.rsq.tab"))
 
-    tab$sd.1=get.sd("cv.rsq.tab")
+    tab$sd.1=cv.sd("cv.rsq.tab")
 
     if(!is.null(x$cv.class.rate.tab)) {
-        if(wide.spacing) tab$"    ClassRate" <- get.field("cv.class.rate.tab")
-        else             tab$"ClassRate"   <- get.field("cv.class.rate.tab")
+        if(wide.spacing) tab$"    ClassRate" <- cv.field("cv.class.rate.tab")
+        else             tab$"ClassRate"   <- cv.field("cv.class.rate.tab")
 
-        tab$sd.2 <- get.sd("cv.class.rate.tab")
+        tab$sd.2 <- cv.sd("cv.class.rate.tab")
     }
     if(wide.spacing)
         tab$"    MaxErr" <- x$cv.maxerr.tab[ilast,]
@@ -456,20 +479,20 @@ print.cv <- function(x) # called from print.earth for cross validated models
     tab$sd <- apply(x$cv.maxerr.tab[-ilast,], 2, sd)
 
     if(!is.null(x$cv.auc.tab)) {
-        tab$"AUC" <- get.field("cv.auc.tab")
-        tab$sd.3  <- get.sd("cv.auc.tab")
+        tab$AUC  <- cv.field("cv.auc.tab")
+        tab$sd.3 <- cv.sd("cv.auc.tab")
     }
     if(!is.null(x$cv.cor.tab)) {
-        tab$"cv.cor.tab" <- get.field("cv.cor.tab")
-        tab$sd.4         <- get.sd("cv.cor.tab")
+        tab$cor.tab <- cv.field("cv.cor.tab")
+        tab$sd.4    <- cv.sd("cv.cor.tab")
     }
     if(!is.null(x$cv.deviance.tab)) {
-        tab$"MeanDev"    <- x$cv.deviance.tab[ilast,]
-        tab$sd.5         <- apply(x$cv.deviance.tab[-ilast,], 2, sd)
-        tab$"CalibInt"   <- get.field("cv.calib.int.tab")
-        tab$sd.6         <- get.sd("cv.calib.int.tab")
-        tab$"CalibSlope" <- get.field("cv.calib.slope.tab")
-        tab$sd.7         <- get.sd("cv.calib.slope.tab")
+        tab$MeanDev    <- x$cv.deviance.tab[ilast,]
+        tab$sd.5       <- apply(x$cv.deviance.tab[-ilast,], 2, sd)
+        tab$CalibInt   <- cv.field("cv.calib.int.tab")
+        tab$sd.6       <- cv.sd("cv.calib.int.tab")
+        tab$CalibSlope <- cv.field("cv.calib.slope.tab")
+        tab$sd.7       <- cv.sd("cv.calib.slope.tab")
     }
     # change "sd.N" to plain "sd"
     names <- names(tab)
@@ -478,10 +501,10 @@ print.cv <- function(x) # called from print.earth for cross validated models
 
     rownames(tab) <- c(colnames(x$fitted.values), "All")
 
-    digits <- min(getOption("digits"), 2)
+    digits <- min(getOption("digits"), 3)
 
     if(NCOL(x$coefficients) == 1)   # single response model?
-        print(tab[1,,drop=FALSE], digits=digits, row.names=FALSE) # don't print "All" row
+        print(tab[1,,drop=FALSE], digits=digits, row.names=FALSE) # skip "All" row
     else                            # multiple response model
         print(tab, digits=digits)
 }
